"""
Module containing default tolerances for transport outputs

All unique {keys:tolerances} for plotting outputs.
Each entry has a unique key and value = {'tolerance':value, 'unit':Unit.unit}
where Unit is an enum class.  If a quantity does not have an associated unit,
unit = null.

Keys were generated by using the tolerance.py script.
See that for command line details.

Thermal Transport Properties
  * SEEBECK_11.OUT
  * ELECTCOND_11.OUT
  * THERMALCOND_11.OUT
  * Z_11.OUT

"""
from excitingtools import Unit

from ..tol_classes import DefaultTolerances, Tol, TolWithMessage


default = DefaultTolerances(integer=Tol(0),
                            float=Tol(1.e-8),
                            energy=Tol(1.e-8, Unit.hartree),
                            temperature=Tol(1.e-8, Unit.kelvin),
                            au=Tol(1.e-8, Unit.au)
                            )

transport_function_11 = {'temperature': default.temperature,
                         'mu': default.energy,
                         're': default.au,
                         'im': default.au
                         }

transport_tolerances = {'SEEBECK_11.OUT': transport_function_11,
                        'ELECTCOND_11.OUT': transport_function_11,
                        'THERMALCOND_11.OUT': transport_function_11,
                        'Z_11.OUT': transport_function_11
                        }